## Functions for working with Lab space and preparing stimuli for sound-color experiment

library(soundgen)
library(seewave)

setwd('/home/allgoodguys/Documents/Studying/Lund_PhD/epistles/010_soundCol_perceptual/stimuli/stimuli_apr13')

## VISUAL
myColors = read.csv('Lab_color_space.csv')[, -1]

# what does the range of rgb colors look like in Lab space? See also https://en.wikipedia.org/wiki/Lab_color_space 
res = 30
idx = seq(0, 1, length.out = res)
rgb_ran = expand.grid(r = idx, g = idx, b = idx)
lab = t(apply(rgb_ran, 1, function(x) {
  convertColor(x, from = 'sRGB', to = 'Lab')
}))
colnames(lab) = c('L', 'a', 'b')
lab = as.data.frame(lab)
lab$rgb = apply(rgb_ran, 1, function(x) rgb(matrix(x, nrow = 1)))

# Plot slices through Lab space at a few levels of brightness
sliceLab = function(constraints = list(a = NULL, b = NULL, L = 70), plusMinus = 2) {
  temp = lab
  label = ''
  if (!is.null(constraints$a)) {
    temp = temp[(temp$a > (constraints$a - plusMinus)) & (temp$a < (constraints$a + plusMinus)), ]
    label = paste0('a = ', constraints$a)
  }
  if (!is.null(constraints$b)) {
    temp = temp[(temp$b > (constraints$b - plusMinus)) & (temp$b < (constraints$b + plusMinus)), ]
    label = paste(label, paste('b =', constraints$b))
  }
  if (!is.null(constraints$L)) {
    temp = temp[(temp$L > (constraints$L - plusMinus)) & (temp$L < (constraints$L + plusMinus)), ]
    label = paste(label, paste('L =', constraints$L))
  }
  
  free_par = c('a', 'b', 'L')[which(!c('a', 'b', 'L') %in% names(constraints))]
  if (length(free_par) < 2) {
    plot(x = as.numeric(unlist(temp[free_par])), 
         y = rep(0, nrow(temp)),
         col = temp$rgb, pch = 16, 
         xlab = free_par, ylab = '', main = label, cex = 5)
  } else {
    plot(as.numeric(unlist(temp[free_par[1]])), 
         as.numeric(unlist(temp[free_par[2]])), 
         col = temp$rgb, pch = 16, 
         main = label, xlab = free_par[1], ylab = free_par[2],
         xlim = range(lab[free_par[1]]), ylim = range(lab[free_par[2]]))
  }
  return(temp)
}
# s70 = sliceLab(constraints = list(L = 70)) 
# s70 = sliceLab(constraints = list(a = 40)) 
# s70 = sliceLab(constraints = list(a = 30, b = 50)) 
# s70 = sliceLab(constraints = list(a = 30, L = 50)) 


## Save a stimulus with a particular Lab color
saveLab = function(col_lab, 
                   filename, 
                   path = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/colors-and-sounds/experiment/stimuli/',
                   width = 800, 
                   height = 800) {
  coord_rgb = convertColor(col_lab, 
                           from = 'Lab', 
                           to = 'sRGB')
  jpeg(filename = paste0(path, filename, '.jpg'), width = width, height = height, units = 'px')
  plot(1:10, 1:10, type='n', yaxt='n', xaxt='n', bty='n', xaxs='i', xlab='', ylab='')
  polygon(x = c(1, 10, 10, 1), y = c(1, 1, 10, 10), border = NA, col = rgb(coord_rgb))
  dev.off()
}
# saveLab(col_lab = c(40, 50, 40), filename = 'red')



## Extra visuals
if (FALSE) {
  # plot the whole Lab space
  rgl::plot3d(lab$a, lab$b, lab$L, col=lab$rgb, size = 10)
  
  # individual colors in Lab space
  rgl::plot3d(myColors$a, myColors$b, myColors$L, col=myColors$color, size=5)
  rgl::text3d(myColors$a, myColors$b, myColors$L, col='black', text=myColors$color)
  
  # individual colors by luminance
  png(filename = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/colors-and-sounds/experiment/analysis/pix/colors_by_luminance.png', width = 20, height = 10, units = 'cm', res = 300)
  plot(x = myColors$L, y = rep(5, nrow(myColors)), 
       col = as.character(myColors$color), pch = 16, cex = 3,
       ylim = c(4, 6), xlab = 'Luminance', ylab = '', bty = 'n', yaxt = 'n')
  text(x = myColors$L, y = rep(rnorm(nrow(myColors), 5.3, .1), nrow(myColors)),
       label = myColors$color, col = 'gray60')
  dev.off()
  
  # individual colors by saturation
  myColors$saturation = sqrt(myColors$a ^ 2 + myColors$b ^ 2)
  png(filename = '/home/allgoodguys/Documents/Studying/Lund_PhD/experiments/colors-and-sounds/experiment/analysis/pix/colors_by_saturation.png', width = 20, height = 10, units = 'cm', res = 300)
  plot(x = myColors$saturation, y = rep(5, nrow(myColors)), 
       col = as.character(myColors$color), pch = 16, cex = 3,
       ylim = c(4, 6), xlab = 'Luminance', ylab = '', bty = 'n', yaxt = 'n')
  text(x = myColors$saturation, y = rep(rnorm(nrow(myColors), 5.3, .1), nrow(myColors)),
       label = myColors$color, col = 'gray60')
  dev.off()
}
